import django_rq
from rq.decorators import job

DEFAULT_QUEUE_NAME = "default"
DEFAULT_REDIS_CONNECTION = django_rq.get_connection()


@job(DEFAULT_QUEUE_NAME, connection=DEFAULT_REDIS_CONNECTION)
def send_sms(*args, **kwargs):
	from smsish.sms import send_sms as _send_sms
	return _send_sms(*args, **kwargs)


@job(DEFAULT_QUEUE_NAME, connection=DEFAULT_REDIS_CONNECTION)
def send_mass_sms(*args, **kwargs):
	from smsish.sms import send_mass_sms as _send_mass_sms
	return _send_mass_sms(*args, **kwargs)



from .base_settings import *

# ======================
# django-smsish settings
# ======================

# Add `smsish` to your INSTALLED_APPS
INSTALLED_APPS += (
    'smsish',
)

# Set `SMS_BACKEND` in your settings.
SMS_BACKEND_CONSOLE = 'smsish.sms.backends.console.SMSBackend'
SMS_BACKEND_DUMMY = 'smsish.sms.backends.dummy.SMSBackend'
SMS_BACKEND_RQ = 'smsish.sms.backends.rq.SMSBackend'
SMS_BACKEND_TWILIO = 'smsish.sms.backends.twilio.SMSBackend'
SMS_BACKEND = SMS_BACKEND_DUMMY

# Set Twilio settings if needed.
# Note: `pip install twilio` to use the Twilio backend.
TWILIO_ACCOUNT_SID = os.getenv("TWILIO_ACCOUNT_SID", None)
TWILIO_AUTH_TOKEN = os.getenv("TWILIO_AUTH_TOKEN", None)
TWILIO_MAGIC_FROM_NUMBER = "+15005550006"  # This number passes all validation.
TWILIO_FROM_NUMBER = os.getenv("TWILIO_FROM_NUMBER", TWILIO_MAGIC_FROM_NUMBER)

RQ_QUEUES = {
    'default': {
        'URL': os.getenv("REDIS_URL", None),
    },
}



from django.conf import settings
from django.core import mail
from django.test import TestCase
from django.test.utils import captured_stdout
from django.test.utils import override_settings

from smsish.sms import send_sms
from smsish.sms import send_mass_sms
from smsish.sms.message import SMSMessage

VALID_FROM_NUMBER = settings.TWILIO_MAGIC_FROM_NUMBER
VALID_TO_NUMBER = settings.TWILIO_MAGIC_FROM_NUMBER


@override_settings(SMS_BACKEND='smsish.sms.backends.dummy.SMSBackend')
class SMSMessageTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def test_create_with_subject_not_allowed(self):
		with self.assertRaises(TypeError):
			SMSMessage("Subject", "Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])

	def test_recipients(self):
		sms = self.sms
		recipients = sms.recipients()
		self.assertEqual(recipients, [VALID_TO_NUMBER])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()

		# Test that one message has been sent.
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()

		# Test that no message has been sent.
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		messageSpec = ("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
		datatuple = (messageSpec for _ in range(10))
		numSent = send_mass_sms(datatuple)
		self.assertEqual(numSent, 10)
		self.assertEqual(len(mail.outbox), 0)


@override_settings(SMS_BACKEND='smsish.sms.backends.console.SMSBackend')
class SendSMSUsingConsoleTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def test_send(self):
		with captured_stdout() as stdout:
			sms = self.sms
			numSent = sms.send()
			self.assertEqual(numSent, 1)
			output = stdout.getvalue()
			self.assertTrue("Subject: None" in output)
			self.assertTrue("From: +15005550006" in output)
			self.assertTrue("To: +15005550006" in output)
			self.assertTrue("Body" in output)

		with captured_stdout() as stdout:
			sms_no_recipients = self.sms_no_recipients
			numSent = sms_no_recipients.send()
			self.assertEqual(numSent, 0)
			self.assertEqual(stdout.getvalue(), "")

	def test_send_sms(self):
		with captured_stdout() as stdout:
			numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
			self.assertEqual(numSent, 1)
			self.assertEqual(len(mail.outbox), 0)
			output = stdout.getvalue()
			self.assertTrue("Subject: None" in output)
			self.assertTrue("From: +15005550006" in output)
			self.assertTrue("To: +15005550006" in output)
			self.assertTrue("Body" in output)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		with captured_stdout() as stdout:
			datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
			numSent = send_mass_sms(datatuple)
			self.assertEqual(numSent, 10)
			self.assertEqual(len(mail.outbox), 0)
			output = stdout.getvalue()
			self.assertTrue("Subject: None" in output)
			self.assertTrue("From: +15005550006" in output)
			self.assertTrue("To: +15005550006" in output)
			self.assertTrue("Body" in output)


@override_settings(SMS_BACKEND='smsish.sms.backends.twilio.SMSBackend')
class SendSMSUsingTwilioTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		from smsish.sms import get_sms_connection
		with get_sms_connection(settings.SMS_BACKEND) as connection:
			datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
			numSent = send_mass_sms(datatuple, connection=connection)
			self.assertEqual(numSent, 10)
			self.assertEqual(len(mail.outbox), 0)


@override_settings(
	SMS_BACKEND='smsish.sms.backends.filebased.SMSBackend',
	EMAIL_FILE_PATH="outbox",)
class SendSMSUsingFilebasedTestCase(TestCase):
	"""
	TODO: Test that files are actually written: see https://github.com/django/django/blob/master/tests/mail/tests.py#L876.
	"""
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		from smsish.sms import get_sms_connection
		with get_sms_connection(settings.SMS_BACKEND) as connection:
			datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
			numSent = send_mass_sms(datatuple, connection=connection)
			self.assertEqual(numSent, 10)
			self.assertEqual(len(mail.outbox), 0)


@override_settings(SMS_BACKEND='smsish.sms.backends.locmem.SMSBackend')
class SendSMSUsingLocmemTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 1)

	def test_send_sms(self):
		numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 1)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		from django.conf import settings
		from smsish.sms import get_sms_connection
		with get_sms_connection(settings.SMS_BACKEND) as connection:
			datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
			numSent = send_mass_sms(datatuple, connection=connection)
			self.assertEqual(numSent, 10)
			self.assertEqual(len(mail.outbox), 10)


@override_settings(SMS_BACKEND='smsish.sms.backends.rq.SMSBackend')
@override_settings(SMSISH_RQ_SMS_BACKEND='smsish.sms.backends.locmem.SMSBackend')
@override_settings(TESTING=True)
class SendSMSUsingRQTestCase(TestCase):
	def setUp(self):
		self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

	def get_new_sms_message(self):
		return SMSMessage(
			"Body",
			VALID_FROM_NUMBER,
			[VALID_TO_NUMBER]
		)

	def test_send(self):
		with self.assertRaises(AssertionError):
			sms = self.get_new_sms_message()
			numSent = sms.send()
			self.assertEqual(numSent, 1)
			self.assertEqual(len(mail.outbox), 1)

	def process_jobs(self):
		from django_rq import get_worker
		get_worker().work(burst=True)

	def test_send_with_connection(self):
		# from django.conf import settings
		from smsish.sms import get_sms_connection
		sms = self.get_new_sms_message()
		with get_sms_connection() as connection:
			jobs = connection.send_messages([sms])
			self.assertEqual(len(jobs), 1)
			# http://python-rq.org/docs/testing/
			# https://github.com/ui/django-rq#testing-tip
			self.assertEqual(len(mail.outbox), 1)
			self.process_jobs()
			for job in jobs:
				self.assertTrue(job.id)
				self.assertEqual(job.args[0].body, sms.body)

	def test_send_sms(self):
		with self.assertRaises(AssertionError):
			numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
			self.assertEqual(numSent, 1)
			self.assertEqual(len(mail.outbox), 1)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_mass_sms(self):
		from django.conf import settings
		from smsish.sms import get_sms_connection
		with self.assertRaises(NotImplementedError):
			with get_sms_connection(settings.SMS_BACKEND) as connection:
				datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
				numSent = send_mass_sms(datatuple, connection=connection)
				self.assertEqual(numSent, 10)
				self.assertEqual(len(mail.outbox), 0)


TEST_SMTP_BACKENDS = False
if TEST_SMTP_BACKENDS:
	@override_settings(
		SMS_BACKEND='smsish.sms.backends.smtp.SMSBackend',
		EMAIL_HOST="127.0.0.1",
		EMAIL_PORT=1025,
		EMAIL_HOST_USER="",
		EMAIL_HOST_PASSWORD="",
		EMAIL_USE_TLS=False,)
	class SendSMSUsingSMTPTestCase(TestCase):
		def setUp(self):
			self.sms = SMSMessage(
				"Body",
				VALID_FROM_NUMBER,
				[VALID_TO_NUMBER]
			)
			self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

		def test_send(self):
			sms = self.sms
			numSent = sms.send()
			self.assertEqual(numSent, 1)

		def test_send_sms(self):
			numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
			self.assertEqual(numSent, 1)

		def test_send_to_nobody(self):
			sms = self.sms_no_recipients
			numSent = sms.send()
			self.assertEqual(numSent, 0)
			self.assertEqual(len(mail.outbox), 0)

		def test_send_mass_sms(self):
			with captured_stdout() as stdout:
				datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
				numSent = send_mass_sms(datatuple)
				self.assertEqual(numSent, 10)
				self.assertEqual(len(mail.outbox), 0)
				output = stdout.getvalue()
				self.assertTrue("Subject: None" in output)
				self.assertTrue("From: +15005550006" in output)
				self.assertTrue("To: +15005550006" in output)
				self.assertTrue("Body" in output)

	@override_settings(SMS_BACKEND='smsish.sms.backends.mailcatcher.SMSBackend')
	class SendSMSUsingMailCatcherTestCase(TestCase):
		def setUp(self):
			self.sms = SMSMessage(
				"Body",
				VALID_FROM_NUMBER,
				[VALID_TO_NUMBER]
			)
			self.sms_no_recipients = SMSMessage("Body", VALID_TO_NUMBER, [])

		def test_send(self):
			sms = self.sms
			numSent = sms.send()
			self.assertEqual(numSent, 1)

		def test_send_sms(self):
			numSent = send_sms("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER])
			self.assertEqual(numSent, 1)

		def test_send_to_nobody(self):
			sms = self.sms_no_recipients
			numSent = sms.send()
			self.assertEqual(numSent, 0)
			self.assertEqual(len(mail.outbox), 0)

		def test_send_mass_sms(self):
			with captured_stdout() as stdout:
				datatuple = (("Body", VALID_FROM_NUMBER, [VALID_TO_NUMBER]) for _ in range(10))
				numSent = send_mass_sms(datatuple)
				self.assertEqual(numSent, 10)
				self.assertEqual(len(mail.outbox), 0)
				output = stdout.getvalue()
				self.assertTrue("Subject: None" in output)
				self.assertTrue("From: +15005550006" in output)
				self.assertTrue("To: +15005550006" in output)
				self.assertTrue("Body" in output)



"""SMS backend that enqueues messages to be sent."""
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

import django_rq

from smsish.sms import get_sms_connection
from smsish.sms.backends.base import BaseSMSBackend


class SMSBackend(BaseSMSBackend):
	"""
	An SMSBackend which sends messages asynchronously with the help of RQ.
	"""
	def __init__(self, backend=None, fail_silently=None, **kwargs):
		super(SMSBackend, self).__init__(fail_silently=fail_silently, **kwargs)

		SMSISH_RQ_SMS_BACKEND = getattr(settings, "SMSISH_RQ_SMS_BACKEND", None)
		if backend is None and SMSISH_RQ_SMS_BACKEND is None:
			raise ImproperlyConfigured("SMSISH_RQ_SMS_BACKEND is required but missing")

		self.backend = SMSISH_RQ_SMS_BACKEND if backend is None else backend

	def send_messages(self, sms_messages):
		"""
		Receives a list of SMSMessage instances and returns a list of RQ `Job` instances.
		"""
		results = []
		for message in sms_messages:
			try:
				assert message.connection is None
			except AssertionError:
				if not self.fail_silently:
					raise
			backend = self.backend
			fail_silently = self.fail_silently
			result = django_rq.enqueue(self._send, message, backend=backend, fail_silently=fail_silently)
			results.append(result)
		return results

	def _send(self, sms_message, backend=None, fail_silently=False):
		send_sms_message(sms_message, backend=backend, fail_silently=fail_silently)


def send_sms_message(sms_message, backend=None, fail_silently=False):
	"""
	Send an SMSMessage instance using a connection given by the specified `backend`.
	"""
	with get_sms_connection(backend=backend, fail_silently=fail_silently) as connection:
		result = connection.send_messages([sms_message])
	return result
